Option Explicit On 
Option Strict On

Imports Microsoft.Win32
Imports System
Imports System.IO

Namespace Edanmo.Net.Mail

    Public NotInheritable Class MailAttachment

        Private _contentDisposition As ContentDisposition = ContentDisposition.Attachment
        Private _contentType As String
        Private _filename As String
        Private _id As String

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String)

            If Not File.Exists(path) Then Throw New FileNotFoundException

            Me.Path = path

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String, ByVal disposition As ContentDisposition)

            Me.New(path)

            _contentDisposition = disposition

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String, ByVal contentType As String)

            Me.New(path)

            _contentType = contentType

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String, ByVal contentType As String, ByVal id As String)

            Me.New(path)

            _contentType = contentType
            _id = id

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String, ByVal disposition As ContentDisposition, ByVal id As String)

            Me.New(path)

            _contentDisposition = disposition
            _id = id

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Initializes a new instance with the specified file.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal path As String, ByVal disposition As ContentDisposition, ByVal id As String, ByVal contentType As String)

            Me.New(path)

            _contentDisposition = disposition
            _contentType = contentType
            _id = id

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the file contents.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Friend Function GetData() As Byte()

            ' Open the file for reading
            Dim file As New IO.FileStream(_filename, IO.FileMode.Open, IO.FileAccess.Read, IO.FileShare.Read)

            Try

                ' Initialize the buffer
                Dim fileData(Convert.ToInt32(file.Length) - 1) As Byte

                ' Read the file
                file.Read(fileData, 0, Convert.ToInt32(file.Length))

                Return fileData

            Finally

                ' Close the file
                file.Close()

            End Try

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Content-Dispotion header of the attachment.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property ContentDisposition() As ContentDisposition
            Get
                Return _contentDisposition
            End Get
            Set(ByVal value As ContentDisposition)
                _contentDisposition = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the Content-Id header of the attachement that can be 
        ''' used in a HTML message to embed the attachment.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property ContentId() As String
            Get
                Return _id
            End Get
            Set(ByVal value As String)
                _id = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the attachment MIME format.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property ContentType() As String
            Get
                Return _contentType
            End Get
            Set(ByVal value As String)
                _contentType = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the attachment file name.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property FileName() As String
            Get
                Return IO.Path.GetFileName(_filename)
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the attachment full path.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Path() As String
            Get
                Return _filename
            End Get
            Set(ByVal value As String)

                If Not File.Exists(value) Then Throw New FileNotFoundException

                _filename = value
                _contentType = GetFileContentType(value)

            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retrieves the attachment MIME type from the registry.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Function GetFileContentType(ByVal fileName As String) As String

            Dim key As RegistryKey

            Try

                key = Registry.ClassesRoot.OpenSubKey(IO.Path.GetExtension(fileName), False)

                Return key.GetValue("Content Type", "application/octet-stream").ToString

            Catch

                Return "application/octet-stream"

            Finally

                If Not key Is Nothing Then key.Close()

            End Try

        End Function

    End Class

End Namespace
