Option Explicit On 
Option Strict On

Imports System
Imports System.Globalization

Namespace Edanmo.Net.Mail

    Public NotInheritable Class MailMessage

        Private _attachments As New MailAttachmentCollection
        Private _headers As New MailHeaderCollection
        Private _message As String
        Private _to As New MailRecipientCollection
        Private _cc As New MailRecipientCollection
        Private _bcc As New MailRecipientCollection
        Private _sender As MailRecipient
        Private _bodyFormat As MailFormat

        Public Sub New()
        End Sub

        Public Sub New(ByVal sender As MailRecipient, ByVal recipients() As MailRecipient, ByVal subject As String, ByVal body As String, ByVal fileAttachments() As String)

            Me.New()

            _sender = sender
            _to.AddRange(recipients)
            Me.Headers.Subject = subject
            _message = body

            If Not fileAttachments Is Nothing Then
                For i As Integer = 0 To fileAttachments.Length - 1
                    _attachments.Add(fileAttachments(i))
                Next
            End If

        End Sub

        Public Sub New(ByVal sender As MailRecipient, ByVal recipient As MailRecipient, ByVal subject As String, ByVal body As String)

            Me.New(sender, New MailRecipient() {recipient}, subject, body, Nothing)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates the email body.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Friend Function GetMessageBody(ByVal toSend As Boolean) As Text.StringBuilder

            Dim msg As New Text.StringBuilder
            Dim hasAttachments As Boolean = _attachments.Count > 0
            Dim messageContentType As String
            Dim boundaryName As String = "NextPart_" & _sender.MailAddress.GetHashCode.ToString("X", CultureInfo.InvariantCulture)

            Try

                ' Add some headers to the collection
                _headers("From") = _sender.ToString
                If _to.Count > 0 Then _headers("To") = _to.ToString
                If _cc.Count > 0 Then _headers("Cc") = _cc.ToString
                _headers("X-Mailer") = Reflection.Assembly.GetExecutingAssembly.FullName


                ' The the mail has attachments change
                ' the content type to multipart/mixed
                If hasAttachments Then

                    ' Get the message content type
                    Select Case _bodyFormat
                        Case MailFormat.Plaintext
                            messageContentType = "text/plain; charset=utf-8"
                        Case MailFormat.Html
                            messageContentType = "text/html; charset=utf-8"
                        Case MailFormat.Other
                            messageContentType = _headers.ContentType
                    End Select

                    If _attachments.IsRelated Then
                        _headers.ContentType = "multipart/related; boundary=""" & boundaryName & """"
                    Else
                        _headers.ContentType = "multipart/mixed; boundary=""" & boundaryName & """"
                    End If

                Else

                    Select Case _bodyFormat
                        Case MailFormat.Plaintext
                            _headers.ContentType = "text/plain; charset=utf-8"
                        Case MailFormat.Html
                            _headers.ContentType = "text/html; charset=utf-8"
                    End Select

                End If

                ' Write the mail headers
                For i As Integer = 0 To _headers.Count - 1
                    msg.AppendFormat("{0}: {1}{2}", _headers.Keys(i), _headers(i), Environment.NewLine)
                Next
                msg.Append(Environment.NewLine)

                ' If the mail has attachments add
                ' the first boundaryName for the message
                If hasAttachments Then

                    ' Write the message preamble
                    msg.AppendFormat("This is a multi-part message in MIME format.{0}", Environment.NewLine)
                    msg.Append(Environment.NewLine)

                    ' Start the message boundaryName
                    msg.AppendFormat("--{0}{1}", boundaryName, Environment.NewLine)

                    ' Write the message content type
                    msg.AppendFormat("Content-Type: {0}{1}", messageContentType, Environment.NewLine)
                    msg.Append(Environment.NewLine)

                End If

                ' Write the message body
                If Not _message Is Nothing Then
                    msg.Append(_message.Replace(Environment.NewLine & "." & Environment.NewLine, Environment.NewLine & ". " & Environment.NewLine))
                    msg.Append(Environment.NewLine)
                End If

                ' Add the attachments
                If hasAttachments Then

                    For i As Integer = 0 To _attachments.Count - 1

                        Dim data() As Byte
                        Dim attachment As Text.StringBuilder

                        Try

                            ' Get the attachment data
                            data = _attachments(i).GetData()

                            attachment = New Text.StringBuilder

                            ' Start the attachment boundaryName
                            attachment.Append(Environment.NewLine)
                            attachment.AppendFormat("--{0}{1}", boundaryName, Environment.NewLine)

                            With _attachments(i)

                                ' Add the attachment headers

                                ' Content-Transfer-Encoding
                                attachment.AppendFormat("Content-Transfer-Encoding: base64{0}", Environment.NewLine)

                                ' Content-Disposition
                                If .ContentDisposition <> ContentDisposition.None Then
                                    attachment.AppendFormat("Content-Disposition: {0}; filename=""{1}""{2}", .ContentDisposition.ToString.ToLower(CultureInfo.InvariantCulture), .FileName, Environment.NewLine)
                                End If

                                ' Content-Type
                                If Not .ContentType Is Nothing AndAlso .ContentType.Length > 0 Then
                                    attachment.AppendFormat("Content-Type: {0}; name=""{1}""{2}", .ContentType, .FileName, Environment.NewLine)
                                End If

                                ' Content-Id
                                If Not _attachments(i).ContentId Is Nothing Then
                                    attachment.AppendFormat("Content-Id: <{0}>{1}", .ContentId, Environment.NewLine)
                                End If

                                attachment.Append(Environment.NewLine)

                            End With

                            ' Write the attachment in Base64
                            attachment.Append(Convert.ToBase64String(data))
                            attachment.Append(Environment.NewLine)


                        Catch ex As Exception

                            ' Write an "error attachment"
                            attachment = New Text.StringBuilder

                            ' Start the attachment boundary 
                            attachment.Append(Environment.NewLine)
                            attachment.AppendFormat("--{0}{1}", boundaryName, Environment.NewLine)

                            ' Add the attachment headers
                            attachment.AppendFormat("Content-Type: text/plain{0}", Environment.NewLine)
                            attachment.AppendFormat("Content-Disposition: inline; filename=""error.txt""{0}", Environment.NewLine)
                            attachment.Append(Environment.NewLine)

                            ' Write the attachment text
                            attachment.Append("There was an error adding the attachment: ")
                            attachment.Append(ex.Message)
                            attachment.Append(Environment.NewLine)

                        Finally

                            ' Append the attachment to the message
                            msg.Append(attachment.ToString)
                            attachment = Nothing

                        End Try

                    Next

                    ' Add the closing boundaryName
                    msg.Append(Environment.NewLine)
                    msg.AppendFormat("--{0}--{1}", boundaryName, Environment.NewLine)

                End If

                If toSend Then
                    msg.Append(".")
                End If

                Return msg

            Finally

                If Not messageContentType Is Nothing Then _headers("Content-Type") = messageContentType
                _headers.Remove("From")
                _headers.Remove("To")

            End Try

        End Function

        Public Overrides Function ToString() As String
            Return GetMessageBody(False).ToString
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the attachment collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Attachments() As MailAttachmentCollection
            Get
                Return _attachments
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the header collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Headers() As MailHeaderCollection
            Get
                Return _headers
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the message body.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Body() As String
            Get
                Return _message
            End Get
            Set(ByVal Value As String)
                _message = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the recipient collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property [To]() As MailRecipientCollection
            Get
                Return _to
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retursn the CC recipient collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property CC() As MailRecipientCollection
            Get
                Return _cc
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the BCC recipient collection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Bcc() As MailRecipientCollection
            Get
                Return _bcc
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the sender recipient.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Sender() As MailRecipient
            Get
                Return _sender
            End Get
            Set(ByVal value As MailRecipient)
                _sender = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the mail subject.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property Subject() As String
            Get
                Return _headers.Subject
            End Get
            Set(ByVal value As String)
                _headers.Subject = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the message format.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Property BodyFormat() As MailFormat
            Get
                Return _bodyFormat
            End Get
            Set(ByVal value As MailFormat)
                _bodyFormat = value
            End Set
        End Property

    End Class

End Namespace
