Option Strict On
Option Explicit On 

Imports System
Imports System.Collections.Specialized
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Text.RegularExpressions

Imports Edanmo.Security.Cryptography

Namespace Edanmo.Net.Mail

    Public NotInheritable Class SmtpMail

        Private Const SMTP_AUTH_OK As String = "334"
        Private Const SMTP_AUTHENTICATED As String = "235"
        Private Const SMTP_OK As String = "250"
        Private Const SMTP_OK_CONNECT As String = "220"
        Private Const SMTP_START_DATA As String = "354"

        Private Shared ReadOnly _mailRegEx As New Regex("^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$", RegexOptions.Compiled)

        Private Shared _credential As New NetworkCredential
        Private Shared _mode As MailSendMode = MailSendMode.ToMailExchanger
        Private Shared _port As Integer = 25
        Private Shared _server As String = "localhost"

        Private Sub New()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Authenticates on the server using CRAM-MD5
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Function AuthCramMd5(ByVal smtp As TcpClient, ByVal user As String, ByVal password As String) As Boolean

            Dim recv As String
            Dim hmacmd5 As New hmacmd5(Encoding.ASCII.GetBytes(password))
            Dim hash As Byte()

            SendData(smtp, "AUTH CRAM-MD5")

            recv = ReceiveData(smtp)

            If recv.StartsWith(SMTP_AUTH_OK) Then

                Try

                    ' Get the base64 challenge and decode it
                    Dim challenge() As Byte = Convert.FromBase64String(recv.Substring(4, recv.IndexOf(Environment.NewLine) - 4))

                    ' Hash the challenge using hmacmd5-MD5
                    hash = hmacmd5.ComputeHash(challenge)

                    ' Create the response
                    Dim response As New StringBuilder

                    ' Add the user name and a white space
                    response.Append(user)
                    response.Append(" ")

                    ' Add the hash in HEX format
                    For i As Integer = 0 To hash.Length - 1
                        response.AppendFormat("{0:x2}", hash(i))
                    Next

                    ' Convert the response to base64 and send it
                    SendData(smtp, Convert.ToBase64String(Encoding.ASCII.GetBytes(response.ToString)))

                    recv = ReceiveData(smtp)
                    If Not recv.StartsWith(SMTP_AUTHENTICATED) Then Throw New MailException(recv)

                Finally

                    DirectCast(hmacmd5, IDisposable).Dispose()

                End Try

            Else

                Throw New MailException(recv)

            End If

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether the mail format is correct.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function CheckMailFormat(ByVal mailAddress As String) As Boolean

            Return _mailRegEx.IsMatch(mailAddress)

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether a mail address has correct format and exists on the domain.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Function IsValidMailAddress(ByVal mailAddress As String) As Boolean

            ' Check mail format first
            If SmtpMail.CheckMailFormat(mailAddress) Then

                ' Get the domain from the mail address
                Dim domain As String = mailAddress.Split("@"c)(1)

                ' Get the mail exchanger servers for the domain
                Dim mx() As MailExchanger = Edanmo.Net.Dns.GetMailExchangers(domain)

                For i As Integer = 0 To mx.Length - 1

                    Dim smtp As TcpClient
                    Dim recv As String

                    Try

                        ' Connect to the server
                        smtp = New TcpClient(mx(i).HostName, 25)
                        smtp.ReceiveTimeout = 15000

                        ' Check connection
                        recv = ReceiveData(smtp)
                        If recv.StartsWith(SMTP_OK_CONNECT) Then

                            ' Say HELLO to server
                            SendData(smtp, "HELO " & System.Net.Dns.GetHostName())

                            ' OK?
                            recv = ReceiveData(smtp)
                            If recv.StartsWith(SMTP_OK) Then

                                ' Send a fake address as the mail sender
                                SendData(smtp, "MAIL FROM: <test@test.com>")

                                ' OK?
                                recv = ReceiveData(smtp)
                                If recv.StartsWith(SMTP_OK) Then

                                    ' Send the mail address to check
                                    SendData(smtp, "RCPT TO: <" & mailAddress & ">")

                                    ' OK?
                                    recv = ReceiveData(smtp)
                                    If recv.StartsWith(SMTP_OK) Then

                                        ' The mail address is OK!
                                        Return True

                                    End If

                                End If

                            End If

                        End If

                    Catch

                        ' Ignoro cualquier excepcion

                    Finally

                        ' Cierro la conexion
                        If Not smtp Is Nothing Then

                            ' Tell the server to close the connection
                            SendData(smtp, "QUIT")

                            ' Close the connection
                            smtp.GetStream.Close()
                            smtp.Close()

                        End If

                        smtp = Nothing

                    End Try

                Next

            End If

            ' The mail address doesn't exists in the domain
            Return False

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Reads data from the connection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Function ReceiveData(ByVal cliente As TcpClient) As String

            Dim stream As NetworkStream = cliente.GetStream
            Dim data(cliente.ReceiveBufferSize - 1) As Byte

            stream.Read(data, 0, cliente.ReceiveBufferSize)

            Return Encoding.ASCII.GetString(data).TrimEnd(Environment.NewLine.Chars(0))

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a MailMessage.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub Send(ByVal message As MailMessage)

            If message.Sender Is Nothing Then Throw New InvalidOperationException(ResourceHelper.GetString("MailNoSender"))
            If message.To.Count = 0 AndAlso message.Bcc.Count = 0 Then Throw New InvalidOperationException(ResourceHelper.GetString("MailNoRecipient"))

            Dim messageBody As String = message.GetMessageBody(True).ToString
            Dim recipients As New MailRecipientCollection

            recipients.Add(message.To)
            recipients.Add(message.CC)
            recipients.Add(message.Bcc)

            If _mode = MailSendMode.ToMailExchanger Then

                For j As Integer = 0 To recipients.Count - 1

                    ' Get the domain from the mail
                    Dim domain As String = recipients(j).MailAddress.Split("@"c)(1)

                    ' Get the SMTP servers for the domain
                    Dim mx() As MailExchanger = Edanmo.Net.Dns.GetMailExchangers(domain)

                    For i As Integer = 0 To mx.Length - 1

                        Try

                            SendMx(mx(i).HostName, message.Sender, recipients(j), messageBody)
                            Exit For

                        Catch

                            ' Rethrow the exception only on the last sever
                            If i = mx.Length - 1 Then Throw

                        End Try

                    Next

                Next

            Else

                SendRelay(_server, _port, _credential, message.Sender, recipients, messageBody)

            End If

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a MailMessage to the specified relay server.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub Send(ByVal message As MailMessage, ByVal server As String, ByVal port As Integer, ByVal credential As NetworkCredential)

            If message.Sender Is Nothing Then Throw New InvalidOperationException(ResourceHelper.GetString("MailNoSender"))
            If message.To.Count = 0 AndAlso message.Bcc.Count = 0 Then Throw New InvalidOperationException(ResourceHelper.GetString("MailNoRecipient"))

            Dim messageBody As String = message.GetMessageBody(True).ToString
            Dim recipients As New MailRecipientCollection

            recipients.Add(message.To)
            recipients.Add(message.CC)
            recipients.Add(message.Bcc)

            SendRelay(server, port, credential, message.Sender, recipients, messageBody)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a MailMessage.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub Send(ByVal message As MailMessage, ByVal credential As NetworkCredential)

            Send(message, _server, _port, credential)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a MailMessage.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub Send(ByVal message As MailMessage, ByVal server As String, ByVal port As Integer)

            Send(message, server, port, _credential)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends data over the connection.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Sub SendData(ByVal cliente As TcpClient, ByVal texto As String)

            Dim stream As NetworkStream = cliente.GetStream
            Dim data() As Byte = Encoding.UTF8.GetBytes(texto & Environment.NewLine)

            stream.Write(data, 0, data.Length)

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a mail to a mail exchanger.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Sub SendMx( _
            ByVal smtpServer As String, _
            ByVal sender As MailRecipient, _
            ByVal recipient As MailRecipient, _
            ByVal message As String)

            Dim smtpClient As TcpClient
            Dim recv As String

            Try

                ' Open the connection to the smtpClient smtpServer
                smtpClient = New TcpClient(smtpServer, 25)

                ' Set the receive timeout to 15 secs
                smtpClient.ReceiveTimeout = 15000

                ' Check for response from the smtpServer
                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK_CONNECT) Then Throw New MailException(recv)

                ' Send HELO command to smtpServer
                SendData(smtpClient, "HELO " & System.Net.Dns.GetHostName())

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Send the sender to the smtpServer
                SendData(smtpClient, "MAIL FROM: <" & sender.MailAddress & ">")

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Send the recipient to the smtpServer
                SendData(smtpClient, "RCPT TO: <" & recipient.MailAddress & ">")

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Start sending the mail data
                SendData(smtpClient, "DATA")

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_START_DATA) Then Throw New MailException(recv)

                ' Send the mail body
                SendData(smtpClient, message)

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Quit
                SendData(smtpClient, "QUIT")

            Finally

                ' Close the connection
                If Not smtpClient Is Nothing Then
                    smtpClient.GetStream.Close()
                    smtpClient.Close()
                End If

                smtpClient = Nothing

            End Try

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Sends a mail to a smtp relay server
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Shared Sub SendRelay( _
                    ByVal smtpServer As String, _
                    ByVal port As Integer, _
                    ByVal credential As NetworkCredential, _
                    ByVal sender As MailRecipient, _
                    ByVal recipient As MailRecipientCollection, _
                    ByVal message As String)

            Dim smtpClient As TcpClient
            Dim recv As String

            Try

                ' Open the connection to the smtpClient smtpServer
                smtpClient = New TcpClient(smtpServer, port)

                ' Set the receive timeout to 15 secs
                smtpClient.ReceiveTimeout = 15000

                ' Check for response from the smtpServer
                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK_CONNECT) Then Throw New MailException(recv)

                ' Send HELO command to smtpServer
                SendData(smtpClient, "EHLO " & System.Net.Dns.GetHostName())

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Try to login if we have an user name
                If Not credential Is Nothing Then

                    ' Authenticate with CRAM-MD5
                    AuthCramMd5(smtpClient, credential.UserName, credential.Password)

                End If

                ' Send the sender to the smtpServer
                SendData(smtpClient, "MAIL FROM: <" & sender.MailAddress & ">")

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Send the recipients to the smtpServer
                For i As Integer = 0 To recipient.Count - 1

                    SendData(smtpClient, "RCPT TO: <" & recipient(i).MailAddress & ">")

                    recv = ReceiveData(smtpClient)
                    If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                Next

                ' Try to send the message body
                SendData(smtpClient, "DATA")

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_START_DATA) Then Throw New MailException(recv)

                ' Send the mail body
                SendData(smtpClient, message)

                recv = ReceiveData(smtpClient)
                If Not recv.StartsWith(SMTP_OK) Then Throw New MailException(recv)

                ' Quit
                SendData(smtpClient, "QUIT")

            Finally

                ' Close the connection
                If Not smtpClient Is Nothing Then
                    smtpClient.GetStream.Close()
                    smtpClient.Close()
                End If

                smtpClient = Nothing

            End Try

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the network credential used to authenticate 
        ''' on the smtp relay server.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Property Credential() As NetworkCredential
            Get
                Return _credential
            End Get
            Set(ByVal value As NetworkCredential)
                _credential = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the smtp relay server port.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Property Port() As Integer
            Get
                Return _port
            End Get
            Set(ByVal value As Integer)
                _port = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the mode used to send the mails.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Property SendMode() As MailSendMode
            Get
                Return _mode
            End Get
            Set(ByVal value As MailSendMode)
                _mode = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the host name of the smtp relay server.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Property Server() As String
            Get
                Return _server
            End Get
            Set(ByVal value As String)
                _server = value
            End Set
        End Property

    End Class

End Namespace

